import { dev } from "$app/environment";
import type { Empty } from "$lib/types.js";
import type { MaybePromise } from "@sveltejs/kit";
import type {
  Adapter,
  RegisteredDatabaseSessionAttributes,
  RegisteredDatabaseUserAttributes,
  Session,
  SessionCookieOptions,
  TimeSpan,
  UserId,
} from "lucia";
import { Lucia } from "lucia";
import type { TokenSet, UserinfoResponse } from "openid-client";

export abstract class LuciaAdapter<
  _SessionAttributes extends Empty = Empty,
  _UserAttributes extends Empty = Empty,
> extends Lucia<_SessionAttributes, _UserAttributes> {
  constructor(
    adapter: Adapter,
    options?: {
      sessionExpiresIn?: TimeSpan;
      sessionCookie?: SessionCookieOptions;
      getSessionAttributes?: (
        databaseSessionAttributes: RegisteredDatabaseSessionAttributes,
      ) => _SessionAttributes;
      getUserAttributes?: (
        databaseUserAttributes: RegisteredDatabaseUserAttributes,
      ) => _UserAttributes;
    },
  ) {
    super(adapter, {
      sessionCookie: {
        attributes: {
          secure: !dev,
        },
        name: "aidc_session",
      },
      ...options,
    });
  }

  abstract getUserId(subject: string): MaybePromise<string | undefined>;
  abstract newUser(
    subject: string,
    userId: string,
    claims: UserinfoResponse,
  ): MaybePromise<string>;
  abstract newSession(userId: UserId, tokenSet: TokenSet): Promise<Session>;
  abstract getIdToken(
    session: Session | null,
  ): MaybePromise<string | undefined>;
}
